// Test code for Ultimate GPS Using Hardware Serial (e.g. GPS Flora or FeatherWing)
//
// This code shows how to listen to the GPS module via polling. Best used with
// Feathers or Flora where you have hardware Serial and no interrupt
//
// Tested and works great with the Adafruit GPS FeatherWing
// ------> https://www.adafruit.com/products/3133
// or Flora GPS
// ------> https://www.adafruit.com/products/1059
// but also works with the shield, breakout
// ------> https://www.adafruit.com/products/1272
// ------> https://www.adafruit.com/products/746
//
// Pick one up today at the Adafruit electronics shop
// and help support open source hardware & software! -ada

void check_receive();

#include <Adafruit_GPS.h>

#include <SPI.h>
#include <RH_RF95.h>

/* for feather m0 */
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 3

#define PACKET_PERIOD 6000   //1000 millliseconds between reports


#define led 13

/* for feather 32u4
  #define RFM95_CS 8
  #define RFM95_RST 4
  #define RFM95_INT 7
*/

// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

// Change to 915.0 or other frequency (434.0), must match RX's freq!
#define RF95_FREQ 915.0

// what's the name of the hardware serial port?
#define GPSSerial Serial1

// Connect to the GPS on the hardware port
Adafruit_GPS GPS(&GPSSerial);

// Set GPSECHO to 'false' to turn off echoing the GPS data to the Serial console
// Set to 'true' if you want to debug and listen to the raw GPS sentences
#define GPSECHO false

int sequence;

void setup()
{
  //while (!Serial);  // uncomment to have the sketch wait until Serial is ready

  // connect at 115200 so we can read the GPS fast enough and echo without dropping chars
  // also spit it out
  Serial.begin(115200);

   pinMode(led, OUTPUT);     

  //9600 NMEA is the default baud rate for Adafruit MTK GPS's- some use 4800
  GPS.begin(9600);
  delay(100);
  // uncomment this line to turn on RMC (recommended minimum) and GGA (fix data) including altitude
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);
  //GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_ALLDATA);
  //GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCONLY);
  //GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ); // 1 Hz update rate

  // Request updates on antenna status, comment out to keep quiet
  // GPS.sendCommand(PGCMD_ANTENNA);

  delay(1000);
  GPSSerial.end();

  Serial1.begin(9600);

  // manual reset
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);

  while (!rf95.init()) {
    Serial.println("LoRa radio init failed");
    while (1);
  }
  Serial.println("LoRa radio init OK!");

  if (!rf95.setFrequency(RF95_FREQ)) {
    Serial.println("setFrequency failed");
    while (1);
  }
  Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);

  //Built fo comfort not for speed
  RH_RF95::ModemConfig modem_config = {
    0x98, // Reg 0x1D: BW=500kHz, Coding=4/8, Header=explicit
    0xc4, // Reg 0x1E: Spread=4096chips/symbol, CRC=enable
    0x0c  // Reg 0x26: LowDataRate=On, Agc=On
  };
  rf95.setModemRegisters(&modem_config);

  // The default transmitter power is 13dBm, using PA_BOOST.
  // If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then
  // you can set transmitter powers from 5 to 23 dBm:
  rf95.setTxPower(23, false);
}


int fillflag = 0;
int pointer = 0;
char in_buffer[200] = "";
int characterCount;
int seqnum;




void loop() // run over and over again
{
  //


  while (!Serial1.available());

  char c = Serial1.read();
  // read data from the GPS in the 'main loop'
  //Serial.print(c);


  switch (c) {
    case '$' :
      pointer = 0;
      in_buffer[pointer++] = c;
      characterCount = 0;
      fillflag = true;
      break;
    case '\n' :
      in_buffer[pointer++] = c;
      characterCount++;
      in_buffer[pointer++] = '\0';

      //Serial.print(in_buffer);

      rf95.waitPacketSent();
      digitalWrite(led, HIGH);
      rf95.send((uint8_t*)in_buffer, sizeof(in_buffer));
      rf95.waitPacketSent();
      digitalWrite(led, LOW);
      //if (rf95.sleep()) Serial.println(" sleeping");  //causing problems
      fillflag = false;

      delay(PACKET_PERIOD);

      pointer = 0;
      characterCount = 0;


    
      break;


    default :

      if (fillflag == true && (characterCount < 119)) {
        in_buffer[pointer++] = c;
        characterCount++;
      }
      break;
  }

}
